/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */


package net.craftforge.essential.context.servlet;

import net.craftforge.essential.controller.*;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A servlet context implementation for forwarding an <code>HttpServletRequest</code>
 * to the framework.
 *
 * @author Christian Bick
 * @since 02.02.2011
 */
public class EssentialHttpServlet extends HttpServlet {

    /**
     * Forwards to doPost(req, resp)
     *
     * @param req The request
     * @param resp The response
     */
	protected void doGet(HttpServletRequest req, HttpServletResponse resp) {
        doPost(req, resp);
	}

    /**
     * Forwards to doPost(req, resp)
     *
     * @param req The request
     * @param resp The response
     */
    protected void doPut(HttpServletRequest req, HttpServletResponse resp) {
        doPost(req, resp);
    }

    /**
     * Forwards to doPost(req, resp)
     *
     * @param req The request
     * @param resp The response
     */
    protected void doDelete(HttpServletRequest req, HttpServletResponse resp) {
        doPost(req, resp);
    }

    /**
     * Forwards to doPost(req, resp)
     *
     * @param req The request
     * @param resp The response
     */
    protected void doOptions(HttpServletRequest req, HttpServletResponse resp) {
        doPost(req, resp);
    }

    /**
     * Forwards to doPost(req, resp)
     *
     * @param req The request
     * @param resp The response
     */
    protected void doHead(HttpServletRequest req, HttpServletResponse resp) {
        doPost(req, resp);
    }

    /**
     * Obtains a controller instance and performs a request on it.
     *
     * @param servletRequest The HTTP servlet request
     * @param servletResponse The HTTP servlet response
     */
    protected void doPost(HttpServletRequest servletRequest, HttpServletResponse servletResponse) {
        try {
            String packageName = getControllerPackage(servletRequest);
            Configuration config = getControllerConfiguration(servletRequest);
            Request req = getControllerRequest(servletRequest);
            Response resp = getControllerResponse(servletResponse);
            Controller.getInstance(packageName, config).perform(req, resp);
        } catch (ControllerException e) {
            getServletContext().log("Failed to respond to request: ", e);
            servletResponse.setStatus(e.getHttpStatusCode().getCode());
        } catch (RuntimeException e) {
            getServletContext().log("Internal error after request: ", e);
            servletResponse.setStatus(500);
        }
    }

    /**
     * Gets the controller package name. This operation may want to make use of
     * request specific information.
     *
     * @param servletRequest The HTTP servlet request
     * @return The controller package name
     */
    protected String getControllerPackage(HttpServletRequest servletRequest) {
        return getInitParameter("package");
    }

    /**
     * Gets the controller configuration.
     *
     * @param servletRequest The http servlet request
     * @return The controller configuration
     */
    protected Configuration getControllerConfiguration(HttpServletRequest servletRequest) {
        return new Configuration();
    }

    /**
     * Gets the controller request from a servlet request.
     *
     * @param servletRequest The HTTP servlet request
     * @return The controller request
     */
    protected Request getControllerRequest(HttpServletRequest servletRequest) {
        return new HttpServletRequestImpl(servletRequest);
    }

    /**
     * Gets the controller response from a servlet response.
     *
     * @param servletResponse The HTTP servlet response
     * @return The controller response
     */
    protected Response getControllerResponse(HttpServletResponse servletResponse) {
        return new HttpServletResponseImpl(servletResponse);
    }
}
